//  Diffusion Client Library for iOS, OS X and tvOS
//
//  Copyright (c) 2015, 2016 Push Technology Ltd., All Rights Reserved.
//
//  Use is subject to license terms.
//
//  NOTICE: All information contained herein is, and remains the
//  property of Push Technology. The intellectual and technical
//  concepts contained herein are proprietary to Push Technology and
//  may be covered by U.S. and Foreign Patents, patents in process, and
//  are protected by trade secret or copyright law.

@import Foundation;
#import <Diffusion/PTDiffusionFeature.h>

@class PTDiffusionBytes;
@class PTDiffusionContent;
@class PTDiffusionMessageStream;
@class PTDiffusionReceiveContext;
@class PTDiffusionSendOptions;
@class PTDiffusionTopicSelector;

@protocol PTDiffusionMessageStreamDelegate;

NS_ASSUME_NONNULL_BEGIN

/**
 @brief The Messaging feature provides a client session with messaging capabilities.
 
 The Messaging feature for a session can be obtained from the session's
 `messaging` property.
 
 @see PTDiffusionSession

 @since 5.6
 */
@interface PTDiffusionMessagingFeature : PTDiffusionFeature

/**
 Send an arbitrary message to the server on a particular topic path.
 
 The path does not need to correspond to an existing topic; however the use of `/` as a hierarchical
 delimiter allows for other sessions to listen to messages from specific paths.
 
 @param topicPath The topic path on which to send the message.
 
 @param content The content of the message.
 
 @param options The send options to use.
 
 @param completionHandler Block to be called asynchronously on success or failure.
 If the operation was successful, the `error` argument passed to the block will be `nil`.
 The completion handler will be called asynchronously on the main dispatch queue.

 @exception NSInvalidArgumentException Raised if any supplied arguments are `nil`.
 
 @deprecated since 5.9, replaced by \ref sendWithTopicPath:value:options:completionHandler:

 @since 5.6
 */
-(void)sendWithTopicPath:(NSString *)topicPath
                 content:(PTDiffusionContent *)content
                 options:(PTDiffusionSendOptions *)options
       completionHandler:(void (^)(NSError * _Nullable error))completionHandler __deprecated;

/**
 Send an arbitrary message to the server on a particular topic path.

 The path does not need to correspond to an existing topic; however the use of `/` as a hierarchical
 delimiter allows for other sessions to listen to messages from specific paths.

 @param topicPath The topic path on which to send the message.

 @param value The content of the message.

 @param options The send options to use.

 @param completionHandler Block to be called asynchronously on success or failure.
 If the operation was successful, the `error` argument passed to the block will be `nil`.
 The completion handler will be called asynchronously on the main dispatch queue.

 @exception NSInvalidArgumentException Raised if any supplied arguments are `nil`.

 @since 5.9
 */
-(void)sendWithTopicPath:(NSString *)topicPath
                   value:(PTDiffusionBytes *)value
                 options:(PTDiffusionSendOptions *)options
       completionHandler:(void (^)(NSError * _Nullable error))completionHandler;

/**
 Send an arbitrary message to the server on a particular topic path using default send options.
 
 The path does not need to correspond to an existing topic; however the use of `/` as a hierarchical
 delimiter allows for other sessions to listen to messages from specific paths.
 
 @param topicPath The topic path on which to send the message.
 
 @param content The content of the message.
 
 @param completionHandler Block to be called asynchronously on success or failure.
 If the operation was successful, the `error` argument passed to the block will be `nil`.
 The completion handler will be called asynchronously on the main dispatch queue.

 @exception NSInvalidArgumentException Raised if any supplied arguments are `nil`.
 
 @deprecated since 5.9, replaced by \ref sendWithTopicPath:value:completionHandler:

 @since 5.6
 */
-(void)sendWithTopicPath:(NSString *)topicPath
                 content:(PTDiffusionContent *)content
       completionHandler:(void (^)(NSError * _Nullable error))completionHandler __deprecated;

/**
 Send an arbitrary message to the server on a particular topic path using default send options.

 The path does not need to correspond to an existing topic; however the use of `/` as a hierarchical
 delimiter allows for other sessions to listen to messages from specific paths.

 @param topicPath The topic path on which to send the message.

 @param value The content of the message.

 @param completionHandler Block to be called asynchronously on success or failure.
 If the operation was successful, the `error` argument passed to the block will be `nil`.
 The completion handler will be called asynchronously on the main dispatch queue.

 @exception NSInvalidArgumentException Raised if any supplied arguments are `nil`.

 @since 5.9
 */
-(void)sendWithTopicPath:(NSString *)topicPath
                   value:(PTDiffusionBytes *)value
       completionHandler:(void (^)(NSError * _Nullable error))completionHandler;

/**
 Add a message stream to receive messages for topics that match the given topic selector.

 When a message is received from the server, it will be passed to all message streams that have
 been added with matching selectors.
 If there is more than one match, the order in which the message streams are called is not defined.
 If there is no matching message stream, the fallback message stream will be called instead.

 @param selector The @ref md_topic_selectors "topic selector" to be evaluated locally.

 @param delegate Receiver of messages on matching message stream events.
 Used by the stream to notify of events but not retained.
 The delegate will be sent messages asynchronously on the main dispatch queue.

 @return A new stream instance configured to notify the delegate of events.
 This does not need to be retained by the caller, but should be if use with the
 `removeMessageStream:` method is required later on.

 @exception NSInvalidArgumentException Raised if any supplied arguments are `nil`.

 @note Since the addition of value streams for version 5.9, this method now
 returns instances of PTDiffusionMessageStream, being a new subclass of
 PTDiffusionStream.

 @since 5.6
 */
-(PTDiffusionMessageStream *)addMessageStreamWithSelector:(PTDiffusionTopicSelector *)selector
                                                 delegate:(id<PTDiffusionMessageStreamDelegate>)delegate;

/**
 Remove a message stream that was previously added using a topic selector or as a fallback.

 @param stream The stream to remove.

 @return `YES` if the stream was removed.

 @note If the supplied `stream` argument is `nil` or the stream has already been removed from
 this feature then calls to this method do nothing and return `NO`.

 @note Since the addition of value streams for version 5.9, this method now
 only accepts instances of PTDiffusionMessageStream, being a new subclass of
 PTDiffusionStream.

 @since 5.6
 */
-(BOOL)removeMessageStream:(nullable PTDiffusionMessageStream *)stream;

/**
 When a message is received from the server, it will be passed on to all message streams that have been
 added with matching selectors. If no message stream is registered with a matching selector, the
 fallback message streams that have been registered using this method will be called instead.

 Zero, one, or many fallback message streams can be set.
 If there is no fallback message stream, messages that match no other will be discarded.

 @param delegate Receiver of messages stream events not handled by an explicitly registered topic selector.
 Used by the stream to notify of events but not retained.
 The delegate will be sent messages asynchronously on the main dispatch queue.

 @return A new message stream instance configured to notify the delegate of events.
 This does not need to be retained by the caller, but should be if use with the
 `removeMessageStream:` method is required later on.

 @exception NSInvalidArgumentException Raised if the supplied `delegate` argument is `nil`.

 @note Since the addition of value streams for version 5.9, this method now
 returns instances of PTDiffusionMessageStream, being a new subclass of
 PTDiffusionStream.

 @since 5.6
 */
-(PTDiffusionMessageStream *)addFallbackMessageStreamWithDelegate:(id<PTDiffusionMessageStreamDelegate>)delegate;

@end

NS_ASSUME_NONNULL_END
