//  Diffusion Client Library for iOS, OS X and tvOS
//
//  Copyright (c) 2016 Push Technology Ltd., All Rights Reserved.
//
//  Use is subject to license terms.
//
//  NOTICE: All information contained herein is, and remains the
//  property of Push Technology. The intellectual and technical
//  concepts contained herein are proprietary to Push Technology and
//  may be covered by U.S. and Foreign Patents, patents in process, and
//  are protected by trade secret or copyright law.

@import Foundation;
#import <Diffusion/PTDiffusionBytes.h>
#import <Diffusion/PTDiffusionTopicsFeature.h>

@class PTDiffusionBinaryDelta;

@protocol PTDiffusionJSONValueStreamDelegate;

NS_ASSUME_NONNULL_BEGIN

/**
 @brief An immutable JSON value with support for both binary and JSON deltas.
 
 JSON is "JavaScript Object Notation", a lightweight data-interchange format.
 See [www.json.org](http://www.json.org).
 
 Internally the value is stored and transmitted not as a JSON string, but in
 CBOR format to reduce memory and network overhead. CBOR (Concise Binary
 Object Representation) is a standardized format for binary representation of
 structured data defined by [RFC 7049](https://tools.ietf.org/html/rfc7049).
 See [www.cbor.io](http://www.cbor.io).

 Each JSON value is represented as a single CBOR data item. CBOR supports
 composite data types just like JSON, so this data item can be an array or a
 map of other data items. The `null` value is represented by the CBOR `Null`
 value.

 @since 5.9
 */
@interface PTDiffusionJSON : PTDiffusionBytes

/**
 Return a JSON object initialised with the given object.
 
 Object hierarchies passed to this method must be constructed from Foundation
 objects which can be logically encoded with CBOR. This provides flexibility
 above and beyond the basic capabilities of a 'pure JSON' encoding as well as
 being less restrictive than the capabilities of NSJSONSerialization.
 
 This means:
 - All objects are instances of `NSString`, `NSData`, `NSNumber`, `NSArray`,
   `NSDictionary` or `NSNull`.
 - Dictionary keys may be instances of any supported type.
 - The top level object may be of any supported type.
 
 Being more permissive than NSJSONSerialization in that:
 - `NSData` instances may be used.
 - Dictionary keys are not restricted to instances of `NSString`.
 - The top level object is not restricted to being a container (`NSArray` or
   `NSDictionary`).
 
 Applications may take advantage of the more permissive nature of the
 underlying CBOR encoding by using these capabilities, assuming:
 1. Encoding to JSON string is not required.
 2. Other client applications, perhaps developed in a different language and
    against a different library, can also support the capabilities.

 @param object The object to be serialized to CBOR and stored internally to
 represent the receiver.
 
 @param error If this method returns `nil` to indicate that an error occured
 then this will be populated with the reason for that failure.

 @exception NSInvalidArgumentException Raised if the `object` argument is `nil`.

 @since 5.9
 */
-(instancetype)initWithObject:(id)object
                        error:(NSError **)error;

/**
 Convenience wrapper around initWithObject:error: that first uses
 NSJSONSerialization to generate the object to be serialized.
 
 @param jsonData A data object containing JSON data. See Apple's
 NSJSONSerialization documentation for detailed information regarding
 supported encodings.
 
 @param error If this method returns `nil` to indicate that an error occured
 then this will be populated with the reason for that failure.

 @exception NSInvalidArgumentException Raised if the `jsonData` argument is `nil`.

 @since 5.9
 */
-(instancetype)initWithJSONData:(NSData *)jsonData
                          error:(NSError **)error;

/**
 Compare the receiver's JSON value with another (e.g. an earlier version) to
 create a binary delta.
 
 @param json The original to which the delta should be able to be applied in
 order to generate the value represented by the receiver.
 
 @param error If an error occurs, upon return contains an `NSError` object
 that describes the problem.
 
 @return A delta representing the difference between json and the receiver,
 or `nil` if there was an error (e.g. either the original or the receiver
 don't represent a valid JSON value).

 @exception NSInvalidArgumentException Raised if the json argument is `nil`.

 @since 5.9
 */
-(nullable PTDiffusionBinaryDelta *)binaryDiffFromJSON:(PTDiffusionJSON *)json
                                                 error:(NSError **)error;

/**
 Apply a binary delta to the receiver's JSON value to create a new value.
 
 @param delta The delta describing the changes to be applied to the receiver's
 value in order to create the value to be returned.
 
 @param error If an error occurs, upon return contains an `NSError` object
 that describes the problem.
 
 @return The new JSON value, or `nil` if there was an error (e.g. the receiver
 does not represent a valid JSON value).
 
 @exception NSInvalidArgumentException Raised if the delta argument is `nil`.

 @since 5.9
 */
-(nullable PTDiffusionJSON *)applyDelta:(PTDiffusionBinaryDelta *)delta
                                  error:(NSError **)error;

/**
 Check whether the receiver is valid.

 @param error If not valid, upon return contains an `NSError` object that
 describes the problem.

 @return `YES` if this instance is valid.

 @since 5.9
 */
-(BOOL)validateWithError:(NSError **)error;

/**
 Creates a Foundation object from the receiver's JSON value.

 @param error If an error occurs, upon return contains an `NSError` object
 that describes the problem.

 @return An object, or `nil` if there was an error (e.g. the receiver doesn't
 represent a valid JSON value).

 @since 5.9
 */
-(nullable id)objectWithError:(NSError **)error;

/**
 Convenience wrapper around objectError: that then uses NSJSONSerialization to
 generate the JSON data.

 @param error If an error occurs, upon return contains an `NSError` object
 that describes the problem.

 @return A data object containing JSON, or `nil` if there was an error (e.g.
 the receive doesn't represent a valid JSON value or cannot be converted to
 pure JSON).
 
 @since 5.9
 */
-(nullable NSData*)JSONDataWithError:(NSError **)error;

/**
 Creates a value stream capable of receiving JSON values.

 @param delegate The object which will handle the incoming stream. A weak
 reference is maintained to this object by the value stream.

 @return A value stream associated with the supplied delegate.

 @exception NSInvalidArgumentException Raised if the delegate argument is `nil`.

 @since 5.9
 */
+(PTDiffusionValueStream *)valueStreamWithDelegate:(id<PTDiffusionJSONValueStreamDelegate>)delegate;

/**
 Compares the receiver to the given JSON.

 @param json The JSON object with which to compare the receiver.

 @return `YES` if the data in json is equal to the contents of the receiver, otherwise `NO`.

 @note Two JSON objects are equal if they are internally represented by the
 same CBOR encoding.

 @since 5.9
 */
-(BOOL)isEqualToJSON:(nullable PTDiffusionJSON *)json;

@end

NS_ASSUME_NONNULL_END
