//  Diffusion Client Library for iOS, OS X and tvOS
//
//  Copyright (c) 2015, 2016 Push Technology Ltd., All Rights Reserved.
//
//  Use is subject to license terms.
//
//  NOTICE: All information contained herein is, and remains the
//  property of Push Technology. The intellectual and technical
//  concepts contained herein are proprietary to Push Technology and
//  may be covered by U.S. and Foreign Patents, patents in process, and
//  are protected by trade secret or copyright law.

@import Foundation;

@class PTDiffusionSessionConfiguration;
@class PTDiffusionTopicsFeature;
@class PTDiffusionMessagingFeature;
@class PTDiffusionPingsFeature;
@class PTDiffusionSecurityFeature;
@class PTDiffusionSessionId;
@class PTDiffusionSessionState;
@class PTDiffusionTopicControlFeature;
@class PTDiffusionTopicUpdateControlFeature;

NS_ASSUME_NONNULL_BEGIN

/**
 @brief A session represents a single connection to a single Diffusion server.
 
 The @ref md_quick_start "Quick Start" guide provides basic instructions on how
 to get connected to a Diffusion server.
 
 @see PTDiffusionSessionStateChange

 @since 5.6
 */
@interface PTDiffusionSession : NSObject

/**
 Starts asynchronously connecting to a Diffusion server using the given session configuration.
 
 @param url The location of the server to connect to. Encapsulates host, port and protocol to use.
 Must be absolute.

 @param configuration The configuration options to connect with.

 @param completionHandler The completion handler to call on connection success or failure.
 The completion handler will be called from the main dispatch queue and will only be called once.
 
 @note Your completion handler _must_ store a strong reference to the session instance in order
 for it to remain open.

 @note The completion handler block is called after the session's state property has been set
 to connected but before those observing via the notification center are informed.

 @since 5.6
 */
+(void)openWithURL:(NSURL *)url
     configuration:(PTDiffusionSessionConfiguration *)configuration
 completionHandler:(void (^)(PTDiffusionSession * _Nullable session, NSError * _Nullable error))completionHandler;

/**
 Starts asynchronously connecting to a Diffusion server using the default session configuration.

 @param url The location of the server to connect to. Encapsulates host, port and protocol to use.
 Must be absolute.

 @param completionHandler The completion handler to call on connection success or failure.
 The completion handler will be called from the main dispatch queue and will only be called once.

 @note Your completion handler _must_ store a strong reference to the session instance in order
 for it to remain open.
 
 @note The completion handler block is called after the session's state property has been set
 to connected but before those observing via the notification center are informed.
 
 @since 5.6
 */
+(void)openWithURL:(NSURL *)url
 completionHandler:(void (^)(PTDiffusionSession * _Nullable session, NSError * _Nullable error))completionHandler;

/**
 The configuration used to open the session.

 @since 5.6
 */
@property(nonatomic, readonly, copy) PTDiffusionSessionConfiguration* configuration;

/**
 The unique identifier for the session as assigned by the server it connects to.
 
 This property is Key-Value Observable with changes being notified on the main dispatch queue.
 */
@property(nonatomic, readonly) PTDiffusionSessionId* sessionId;

/**
 The current state of the session.
 
 This property is Key-Value Observable with changes being notified on the main dispatch queue.
 KVO notifications are sent before the invocation of any callback handler blocks.
 
 @note Session state can also be observed using the notification center.
 See PTDiffusionSessionStateChange for details.

 @since 5.7
 */
@property(nonatomic, readonly) PTDiffusionSessionState* state;

/**
 Close the session. Once closed a session cannot be restarted.

 @since 5.6
 */
-(void)close;

/**
 The Topics feature provides the capability to receive streamed topic updates and/or
 fetch the state of topics.

 @since 5.6
 */
@property(nonatomic, readonly) PTDiffusionTopicsFeature* topics;

/**
 The Topic Control features provides the capability to manage topics.
 
 @since 5.7
 */
@property(nonatomic, readonly) PTDiffusionTopicControlFeature* topicControl;

/**
 The Topic Update Control feature provides the capability to update topics.
 
 @since 5.7
 */
@property(nonatomic, readonly) PTDiffusionTopicUpdateControlFeature* topicUpdateControl;

/**
 The Messaging feature provides messaging capabilities.

 @since 5.6
 */
@property(nonatomic, readonly) PTDiffusionMessagingFeature* messaging;

/**
 The Security feature provides a client session with the ability to change
 the associated principal.
 
 @since 5.6
 */
@property(nonatomic, readonly) PTDiffusionSecurityFeature* security;

/**
 The Pings feature provides a client session with the ability to test the
 roundtrip time associated with sending a request to the server and receiving
 the associated response.
 
 @since 5.7
 */
@property(nonatomic, readonly) PTDiffusionPingsFeature* pings;

@end

NS_ASSUME_NONNULL_END
